/**
 * Film data model.
 * @author Alejandro Arnáiz González, deathnew[at]gmail.com
 * @author Diego Crenes Carrera, dcrenes[at]gmail.com
 */
define('models/film/film',
		['app',
		 'shared/urlcatalog',
		 'shared/keys',
		 'utils/validationutils',
		 'nls/i18nhelper',
		 'ember',
		 'cordova',
		 'models/user/user'],
		 function(Filmeet, UrlCatalog, Keys, ValidationUtils, I18nHelper){
	/*
	 * Creating Filmeet.Film class based on Ember.Object
	 */
	Filmeet.Film = Ember.Object.extend({
		/** Film sid */
		sid: null,
		/** Owner sid */
		ownerSid: null,
		/** Indicates if film is at the cinema */
		inCinema: false,
		/** Film name */
		name: null,
		/** Film genre */
		genre: null,
		/** Film synopsis */
		synopsis: null,
		/** Film premiere year */
		premiere: null
	});
	/*
	 * Reopening class for coding static properties and functions
	 */
	Filmeet.Film.reopenClass({
		// Functions
		/** Gets a film from its id */
		get: function(filmSid){
			var film = Filmeet.Film.create();
			$.ajax({
				type: 'GET',
				cache: false,
				url: UrlCatalog.get('getFilmFromSid')+filmSid,
				dataType:'json',
				success: function(data){
					film.set('sid', data.sid);
					film.set('inCinema', data.inCinema);
					film.set('name', data.name);
					film.set('genre', data.genre);
					film.set('synopsis', data.synopsis);
					film.set('premiere', data.premiere);
				}
			});
			return film;
		},
		/** Gets the user film library filtered */
		getMyFilms: function (){
			var myFilmsList = []
			$.ajax({
				type: 'GET',
				cache: false,
				url: UrlCatalog.get('getMyFilms'),
				data: {'hash': localStorage.getItem(Keys.authorizationHash)},
				dataType:'json',
				success: function(data){
					data.forEach(function(film){
						myFilmsList.addObject(Filmeet.Film.create({
							sid: film.sid,
							inCinema: film.inCinema,
							name: film.name,
							genre: film.genre,		
							synopsis: film.synopsis,
							premiere: film.premiere
						}));
					});
				}
			});
			return myFilmsList;
		},
		/** Gets the user film library filtered */
		getMyFilmsWithFilters: function (gotIt, seeWannaSee){
			var filteredList = []
			$.ajax({
				type: 'GET',
				cache: false,
				url: UrlCatalog.get('getMyFilmsWithFilters'),
				data: {'gotIt': gotIt,'seeWannaSee': seeWannaSee, 'hash': localStorage.getItem(Keys.authorizationHash)},
				dataType:'json',
				success: function(data){
					data.forEach(function(film){
						filteredList.addObject(Filmeet.Film.create({
							sid: film.sid,
							inCinema: film.inCinema,
							name: film.name,
							genre: film.genre,		
							synopsis: film.synopsis,
							premiere: film.premiere
						}));
					});
				}
			});
			return filteredList;
		},
		/** Creates a new film */
		saveNew: function(name, genre, synopsis, premiere, inCinema, successCallBack){
			var emptyName = ValidationUtils.isEmptyOrNullString(name);
			var premiereIsNumber = ValidationUtils.isNumber(premiere);
			if(!emptyName && premiereIsNumber){
				$.ajax({
					type: 'POST',
					cache: false,
					url: UrlCatalog.get('saveNewFilm'),
					data: {'inCinema': inCinema, 'name': name, 'genre': genre, 'synopsis': synopsis, 'premiere': premiere, 'hash': localStorage.getItem(Keys.authorizationHash)},
					dataType:'json',
					success: successCallBack
				});
			} else {
				var errorMsg = "";
				if(emptyName){
					errorMsg = I18nHelper.getValueFromKey('errorNoFilmName');
					if(!premiereIsNumber){
						errorMsg = errorMsg + "\n";
					}
				}
				if(!premiereIsNumber){
					errorMsg = errorMsg + I18nHelper.getValueFromKey('errorInvalidPremiereNumber');
				}
				
				navigator.notification.alert(errorMsg, function(){}, I18nHelper.getValueFromKey('errors'));
			}
		},
		/** Searches films with the matching criteria */
		search: function(searchedText, notGotIt){
			var searchResult = [];
			
			$.ajax({
				type: 'GET',
				cache: false,
				url: UrlCatalog.get('search'),
				data: {'words': searchedText, 'notAdded': notGotIt, 'hash': localStorage.getItem(Keys.authorizationHash)},
				dataType:'json',
				success: function(data){
					data.forEach(function(film){
						searchResult.addObject(Filmeet.Film.create({
							sid: film.sid,
							inCinema: film.inCinema,
							name: film.name,
							genre: film.genre,		
							synopsis: film.synopsis,
							premiere: film.premiere
						}));
					});
				}
			});			
			
			return searchResult;
		},
		getSevenLastAdded: function(){
			var sevenLastAddedFilms = [];
			
			$.ajax({
				type: 'GET',
				cache: false,
				url: UrlCatalog.get('lastSevenFilms'),
				data: {'hash': localStorage.getItem(Keys.authorizationHash)},
				dataType:'json',
				success: function(data){
					data.forEach(function(film){
						sevenLastAddedFilms.addObject(Filmeet.Film.create({
							sid: film.sid,
							inCinema: film.inCinema,
							name: film.name,
							genre: film.genre,		
							synopsis: film.synopsis,
							premiere: film.premiere
						}));
					});
				}
			});			
			
			return sevenLastAddedFilms;
		},
		getCandidates: function(searchedText, meetingSid){
			var filmCandidates = [];
			$.ajax({
				type: 'GET',
				cache: false,
				url: UrlCatalog.get('getCandidates'),
				data: {'words': searchedText, 'meetingSid': meetingSid, 'hash': localStorage.getItem(Keys.authorizationHash)},
				dataType:'json',
				success: function(data){
					data.forEach(function(film){
						filmCandidates.addObject(Filmeet.Film.create({
							sid: film.sid,
							name: film.name	
						}));
					});
				}
			});
			return filmCandidates;
		}
	});
});